/*
Save Point By The Lake
by Joo Won Park (2024)
www.joowonpark/savepoint
for laptop ensemble

1. Put the downloaded SavePoint folder in the Desktop. Do not revove .wav files from the folder
2. Open this file in SuperCollider (http://supercollider.github.io/download)
3. In Menu, select Language -> Evaluate File. A GUI window will appear after a few seconds
4. Follow the instructions in the score

*/
s.waitForBoot{
~loadsounds={
		//edit the next line if SuperCollider cannot find the files
	~folderlocation = "~/Desktop/SavePoint".standardizePath;
	~files = [
		~folderlocation +/+ "Bb_44Hz_24bit.wav",
		~folderlocation +/+ "BbHigh_44Hz_24bit.wav",
		~folderlocation +/+ "BbLow_44Hz_24bit.wav",
		~folderlocation +/+ "G_44Hz_24bit.wav",
		~folderlocation +/+ "VoxPad_44Hz_24bit_Bb.wav",
	];

	~bufBb=Buffer.read(s,~files.at(0));
	~bufBb_High=Buffer.read(s,~files.at(1));
	~bufBb_Low=Buffer.read(s,~files.at(2));
	~bufG=Buffer.read(s,~files.at(3));
	~bufVox=Buffer.read(s,~files.at(4),action:{"-------done reading buffer-------".postln});
};

~functions={
	~key=0;//Original: Bb
	~dryonoff=0; //caps lock
	~spacebar=32;// 32 for mac

	SynthDef("EndPad",{
		arg amp,interval,dry,gate=1;
		var sound,sound2,sound3, filter,env,rate,submix,mix;
		rate=interval.midiratio;
		sound=PlayBuf.ar(~bufVox.numChannels,~bufVox.bufnum,rate,doneAction:2)*0.33;
		sound2=PlayBuf.ar(~bufVox.numChannels,~bufVox.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		sound3=PlayBuf.ar(~bufVox.numChannels,~bufVox.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		submix=(sound+sound2+sound3)*amp;

		env=Env.asr(Rand(10.0,14.0),1,Rand(4.0,8.0));
		env=EnvGen.ar(env,gate,doneAction:2);

		filter=LPF.ar(submix,10+(env*8000));

		mix=filter*env;
		Out.ar(0,mix*dry);
		Out.ar(12,mix*(1-dry));
	}).add;

	SynthDef("ToneFloor",{
		arg amp,freq,gate=1;
		var sound,lfo,submix,filter,mix,env;
		//sound=BrownNoise.ar;
		sound=LFTri.ar([freq,freq*Rand(0.999,1.001)].midicps);
		lfo=SinOsc.ar(Rand(0.03,0.06),Rand(0,pi));
		submix=((sound.sum*lfo)+sound)*0.5;
		filter=BPF.ar(submix,Rand(150,300.0),Rand(0.4,0.7));
		env=Env.asr(Rand(7.0,10.0),1,Rand(4.0,8.0));
		env=EnvGen.ar(env,gate,doneAction:2);
		mix=filter.dup*amp*env;

		Out.ar(0,mix*0.1);
		Out.ar(12,mix);
	}).add;

	SynthDef("NoiseFloor",{
		arg amp,gate=1;
		var sound,lfo,submix,filter,mix,env;
		//sound=BrownNoise.ar;
		sound=LFNoise0.ar(Rand(800,1400));
		lfo=SinOsc.ar(Rand(0.03,0.06),Rand(0,pi),0.6);
		submix=((sound*lfo)+sound)*0.5;
		filter=BPF.ar(submix,Rand(150,300.0),Rand(0.4,0.7));
		env=Env.asr(Rand(2.0,4.0),1,Rand(4.0,8.0));
		env=EnvGen.ar(env,gate,doneAction:2);
		mix=filter.dup*amp*env;

		Out.ar(0,mix*0);
		Out.ar(12,mix);
	}).add;

	SynthDef("PianoBb",{
		arg amp,interval,dry;
		var sound,sound2,sound3, rate,mix;
		rate=interval.midiratio;
		sound=PlayBuf.ar(~bufBb.numChannels,~bufBb.bufnum,rate,doneAction:2)*0.33;
		sound2=PlayBuf.ar(~bufBb.numChannels,~bufBb.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		sound3=PlayBuf.ar(~bufBb.numChannels,~bufBb.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		mix=(sound+sound2+sound3)*amp;
		Out.ar(0,mix*dry);
		Out.ar(12,mix*(1-dry));
	}).add;

	SynthDef("PianoBbL",{
		arg amp,interval,dry;
		var sound,sound2,sound3, rate,mix;
		rate=interval.midiratio;
		sound=PlayBuf.ar(~bufBb_Low.numChannels,~bufBb_Low.bufnum,rate,doneAction:2)*0.33;
		sound2=PlayBuf.ar(~bufBb_Low.numChannels,~bufBb_Low.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		sound3=PlayBuf.ar(~bufBb_Low.numChannels,~bufBb_Low.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		mix=(sound+sound2+sound3)*amp;
		Out.ar(0,mix*dry);
		Out.ar(12,mix*(1-dry));
	}).add;


	SynthDef("PianoBbH",{
		arg amp,interval,dry;
		var sound,sound2,sound3, rate,mix;
		rate=interval.midiratio;
		sound=PlayBuf.ar(~bufBb_High.numChannels,~bufBb_High.bufnum,rate,doneAction:2)*0.33;
		sound2=PlayBuf.ar(~bufBb_High.numChannels,~bufBb_High.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		sound3=PlayBuf.ar(~bufBb_High.numChannels,~bufBb_High.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		mix=(sound+sound2+sound3)*amp;
		Out.ar(0,mix*dry);
		Out.ar(12,mix*(1-dry));
	}).add;

	SynthDef("PianoBb_Pop",{
		arg amp,interval,dry;
		var sound,sound2,sound3, rate,mix;
		rate=interval.midiratio;
		sound=PlayBuf.ar(~bufBb.numChannels,~bufBb.bufnum,rate,doneAction:2)*0.33;
		sound2=PlayBuf.ar(~bufBb.numChannels,~bufBb.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		sound3=PlayBuf.ar(~bufBb.numChannels,~bufBb.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		mix=(sound+sound2+sound3)*amp*(LFNoise0.kr(Rand(17,28)).abs);
		Out.ar(0,mix*dry);
		Out.ar(12,mix*(1-dry));
	}).add;

	SynthDef("PianoG",{
		arg amp,interval,dry;
		var sound,sound2,sound3, rate,mix;
		rate=interval.midiratio;
		sound=PlayBuf.ar(~bufG.numChannels,~bufG.bufnum,rate,doneAction:2)*0.33;
		sound2=PlayBuf.ar(~bufG.numChannels,~bufG.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		sound3=PlayBuf.ar(~bufG.numChannels,~bufG.bufnum,rate*Rand(0.985,1.009),doneAction:2)*0.3;
		mix=(sound+sound2+sound3)*amp;
		Out.ar(0,mix*dry);
		Out.ar(12,mix*(1-dry));
	}).add;


	SynthDef("TheLake",{
		arg amp,cf,gate=1;
		var input, sound,filter,lowcut,mix,env;
		input=In.ar(12,2);
		filter=LPF.ar(input,cf+100);
		12.do({ filter = AllpassC.ar(filter, 0.043,[0.04.rand,0.041.rand],[3,3]) });
		lowcut=LPF.ar(filter,1000);
		env=Env.asr(Rand(7.0,10.0),1,Rand(4.0,8.0));
		env=EnvGen.ar(env,gate,doneAction:2);
		mix=lowcut*amp*env;

		Out.ar(0,mix);
	}).add;

	~repeatnote={
		arg howmany,amp,note,overtone;
		Routine({
			howmany.do{
				Synth("PianoBb_Pop",[\amp,amp*rrand(0.5,1.0),\interval,
					note+[12,0,-12,12,0,-12,12,0,-12,overtone].choose,\dry,~dryonoff]);
				rrand(0.02,0.1).wait;
			}
		}).play;
	};

	~endpad={
		arg fourth;
		~end=Synth("EndPad",[\amp,0.85,\interval,fourth,\dry,1]);
		~end2=Synth("EndPad",[\amp,0.85,\interval,fourth+5,\dry,1]);
	};

	~whichnote={
		arg transpose;
		[0,2,4,5,7,9,10].choose+[-12,0,12].choose+transpose+~key
	};

	//key to synthdef assignment
	~qwer={Synth("PianoBbH",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-12),\dry,0])};
	~qwerC={Synth("PianoBbH",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-12),\dry,1])};
	~tyui={Synth("PianoBbH",[\amp,rrand(0.4,0.9),\interval,~whichnote.(0),\dry,0])};
	~tyuiC={Synth("PianoBbH",[\amp,rrand(0.4,0.9),\interval,~whichnote.(0),\dry,1])};
	~op={Synth("PianoBbH",[\amp,rrand(0.8,1.2),\interval,~whichnote.(12),\dry,0])};
	~opC={Synth("PianoBbH",[\amp,rrand(0.6,0.9),\interval,~whichnote.(12),\dry,1])};

	~asd={Synth("PianoBb",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-24),\dry,0])};
	~asdC={Synth("PianoBb",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-24),\dry,1])};
	~fgh={Synth("PianoBb",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-12),\dry,0])};
	~fghC={Synth("PianoBb",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-12),\dry,1])};
	~jkl={Synth("PianoBb",[\amp,rrand(0.4,0.9),\interval,~whichnote.(0),\dry,0])};
	~jklC={Synth("PianoBb",[\amp,rrand(0.4,0.9),\interval,~whichnote.(0),\dry,1])};

	~zx={Synth("PianoBb_Pop",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-24),\dry,0])};
	~zxC={Synth("PianoBb_Pop",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-24),\dry,1])};
	~cvb={Synth("PianoBb_Pop",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-12),\dry,0])};
	~cvbC={Synth("PianoBb_Pop",[\amp,rrand(0.4,0.9),\interval,~whichnote.(-12),\dry,1])};
	~nm={Synth("PianoBb_Pop",[\amp,rrand(0.4,0.9),\interval,~whichnote.(0),\dry,0])};
	~nmC={Synth("PianoBb_Pop",[\amp,rrand(0.4,0.9),\interval,~whichnote.(0),\dry,1])};

	//~repeatnote.(howmany,amp,note,overtone
	~number123={~repeatnote.(rrand(5,10),rrand(0.3,0.7),~whichnote.(-12),[5,7,10].choose);};
	~number4567={~repeatnote.(rrand(10,20),rrand(0.3,0.7),~whichnote.(0),[5,7,10].choose);};
	~number890={~repeatnote.(rrand(20,30),rrand(0.3,0.7),~whichnote.(12),[5,7,10].choose);};

	//space bar
	~space={Synth("PianoBbL",[\amp,rrand(0.6,0.9),\interval,~whichnote.(0),\dry,rrand(0.0,0.2)])};
};

~interface={
	//GUI
	~hor=Window.screenBounds.width;
	~ver=Window.screenBounds.height*0.9;
	~thegui=Window("Save Point Byt The Lake (2024)", Rect(~hor*0.01,~ver*0.1, ~hor*0.75,~ver*0.5)).front;
	~thegui.background=Color(1.0.rand,1.0.rand,1.0.rand,rrand(0.7,0.99));

	//text input/monitor
	~thetext=TextField(~thegui, Rect(~hor*0.01,~ver*0.01, ~hor*0.45,~ver*0.125));
	~thetext.font=Font("Times", 30);
	~thetext.background=Color(1,1,1);
	~thetext.keyDownAction = {
		arg view, char, modifiers, unicode, keycode;
		var notefreq;
		{~thetext.background=Color(1.0.rand,1.0.rand,1.0.rand,rrand(0.0,0.5))}.defer;
		{~thegui.background=Color(1.0.rand,1.0.rand,1.0.rand,1.0.rand)}.defer;

		//[char,modifiers, unicode].postln;
		switch (unicode,
			//QWERTY
			113,{~qwer.value},119,{~qwer.value},101,{~qwer.value},114,{~qwer.value},116,{~tyu.value},
			121,{~tyui.value},117,{~tyui.value},105,{~tyui.value},111,{~op.value},112,{~op.value},
			91,{},93,{},
			//ASDF
			97,{~asd.value},115,{~asd.value},100,{~asd.value},102,{~fgh.value},103,{~fgh.value},
			104,{~fgh.value},106,{~jkl.value},107,{~jkl.value},108,{~jkl.value},
			59,{},39,{},
			//ZXCV
			122,{~zx.value},120,{~zx.value},99,{~cvb.value},118,{~cvb.value},98,{~cvb.value},
			110,{~nm.value},109,{~nm.value},
			44,{},46,{},47,{},
			//numbers
			49,{~number123.value},50,{~number123.value},51,{~number123.value},52,{~number4567.value},53,{~number4567.value},
			54,{~number4567.value},55,{~number4567.value},56,{~number890.value},57,{~number890.value},48,{~number890.value},
			//space bar
			~spacebar,{~space.value;~thetext.string="";},
			//CAPS
			//QWERTY
			81,{~qwerC.value},87,{~qwerC.value},69,{~qwerC.value},82,{~qwerC.value},84,{~tyuC.value},
			89,{~tyuiC.value},85,{~tyuiC.value},73,{~tyuiC.value},79,{~opC.value},80,{~opC.value},
			91,{},93,{},
			//ASDF
			65,{~asdC.value},83,{~asdC.value},68,{~asdC.value},70,{~fghC.value},71,{~fghC.value},
			72,{~fghC.value},74,{~jklC.value},75,{~jklC.value},76,{~jklC.value},
			59,{},39,{},
			//ZXCV
			90,{~zxC.value},88,{~zxC.value},67,{~cvbC.value},86,{~cvbC.value},66,{~cvbC.value},
			78,{~nmC.value},77,{~nmC.value},
			44,{},46,{},47,{}
			;
			//other

	)};

	~wavebutton= Button(~thegui,Rect(~hor*0.01,~ver*0.16,~hor*0.2,~ver*0.1));
	~wavebutton.font= Font("Monaco", 20);
	~wavebutton.states= [["Start"],["Stop"],["Reset"]];
	~wavebutton.action = {
		arg current;
		current.value.postln;
		if (current.value==1,{
			~thelake=Synth("TheLake",[\amp,1.0,\cf,2000,\depth,10]);
			~water=Synth("NoiseFloor",[\amp,0.03]);
			~water2=Synth("NoiseFloor",[\amp,0.02]);
			~thetext.string="****CLICK HERE TO ACTIVATE KEYS****";
		});
		if (current.value==2,{
			~water.release(rrand(9.0,11.0));~water2.release(rrand(10.0,14.0));~thelake.release(rrand(10.0,15.0));
			~thetext.string="****CLICK HERE TO ACTIVATE KEYS****";
		});
		if (current.value==0,{
			~notewave.release(0.1);~notewave2.release(0.1);
			~end.release(1);~end2.release(1);
		})
		//if(view.value == 3) { a.states =  [[["yes", "no"].choose, Color.grey, Color.white]] }
	};

	~wavebutton2= Button(~thegui,Rect(~hor*0.26,~ver*0.16,~hor*0.2,~ver*0.1));
	~wavebutton2.font= Font("Monaco", 20);
	~wavebutton2.states= [["Pitched Drone ON"],["Pitched Drone Off"],["Pitched Drone Reset"]];
	~wavebutton2.action = {
		arg current;
		current.value.postln;
		if (current.value==1,{
			~notewave=Synth("ToneFloor",[\amp,0.03,\freq,(58-12+5)]);
			~notewave2=Synth("ToneFloor",[\amp,0.02,\freq,(58-12+7)]);
			~thetext.string="****CLICK HERE TO ACTIVATE KEYS****";
		});
		if (current.value==2,{
			~notewave.release(20);~notewave2.release(15);
			~thetext.string="****CLICK HERE TO ACTIVATE KEYS****";
		});
		//if(view.value == 3) { a.states =  [[["yes", "no"].choose, Color.grey, Color.white]] }
	};

	~padbutton= Button(~thegui,Rect(~hor*0.01,~ver*0.3,~hor*0.45,~ver*0.1));
	~padbutton.font= Font("Monaco", 20);
	~padbutton.states= [["End Pad ON"],["End Pad Off (if not done already)"],["End Pad Reset"]];
	~padbutton.action = {
		arg current;
		current.value.postln;
		if (current.value==1,{
			~endpad.([-5,0,0,0,5,10].choose+([-12,0,0,0,-12].choose));
			~thetext.string="****CLICK HERE TO ACTIVATE KEYS****";
		});
		if (current.value==2,{
			~end.release(rrand(10.0,15.0));~end2.release(rrand(10.0,15.0));
			~thetext.string="****CLICK HERE TO ACTIVATE KEYS****";
		});
		//if(view.value == 3) { a.states =  [[["yes", "no"].choose, Color.grey, Color.white]] }
	};

					~instruction=StaticText(~thegui,Rect(~hor*0.5,~ver*0.01,~hor*0.2,~ver*0.1));
		~instruction.font= Font("Monaco", 13);
		~instruction.string=
		"by Joo Won Park ®2024
";


	~thegui.onClose={
		s.freeAll;"closed".postln;
		~notewave.release(0.1);~notewave2.release(0.1);
		~end.release(0.1);~end2.release(0.2);
	}
}; //end of GUI

CmdPeriod.doOnce({ ~thegui.close });

SystemClock.sched(0,{~loadsounds.();"loading samples".postln;});
SystemClock.sched(1,{~functions.();"loading synthdefs and functions".postln;});
SystemClock.sched(1.5,{{~interface.()}.defer;"loading GUI".postln});

};//end of waitforBoot


